import logging, sys, re
from colorama import Fore, Style
import coloredlogs, tailer
from functools import wraps
import inspect

# print('hello worlds 2')

########################################################################################################################
# Create colors for the output
class MyFormatter(logging.Formatter):
    prefix = "%(asctime)s "
    FORMATS = {
        logging.ERROR: Fore.RED + prefix + Style.RESET_ALL + "%(message)s", 
        logging.DEBUG: Fore.BLUE + prefix + Style.RESET_ALL + "%(message)s",
        logging.WARNING: Fore.YELLOW + prefix + Style.RESET_ALL + "%(message)s",
        "DEFAULT": Fore.CYAN + prefix + Style.RESET_ALL + "%(message)s",
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno, self.FORMATS['DEFAULT'])
        formatter = logging.Formatter(log_fmt,  datefmt='%Y-%m-%d,%H:%M:%S')
        return formatter.format(record) 


########################################################################################################################
# Log to console and to file
class MCLogger(object):
	log_inst = None 
	logfilepath = ""
	config = []

	# def read_file(last_n_rows):
	def __init__(self, out_filename):
		self._filename = out_filename

		self.log_inst = self._setupLogger()

	################################################################################################
	# Get last n_rows from the log
	def read_log_file(self, last_n_rows = 20):
		with open(self._filename, 'r') as file:
			last_lines = tailer.tail(file, last_n_rows)
		return last_lines
	
	################################################################################################
	# Get last n_rows from the log
	def read_log_file_as_text(self, last_n_rows = 20):
		last_lines = self.read_log_file(last_n_rows)
		for index, line in enumerate(last_lines):
			last_lines[index] = re.sub(r'\x1b\[.+?m', '', last_lines[index], flags=re.MULTILINE )
		return last_lines

	################################################################################################
	# Configure logger
	def _setupLogger(self):
		log_inst = logging.getLogger(__name__)

		custom_formatter = MyFormatter()
		log_inst.setLevel(logging.DEBUG)

		fileHandler = logging.FileHandler( self._filename)
		fileHandler.setFormatter(custom_formatter)
		log_inst.addHandler(fileHandler)

		consoleHandler = logging.StreamHandler(sys.stdout)
		consoleHandler.setFormatter(custom_formatter)
		log_inst.addHandler(consoleHandler)
		log_inst.read_log_file_as_text = self.read_log_file_as_text
		log_inst.read_log_file  = self.read_log_file  
		return log_inst

		# return self.log_inst

	################################################################################################
	# Main function to call
	# def getLogger(self): 
	# 	if self.log_inst	== None:
	# 		self.log_inst	= self._setupLogger( )
	# 	return self.log_inst	 	

	def logfunc_loc(self, original_func): 
		# breakpoint()
		def wrapper_func( *args, **kwargs):	 
			
			color = Fore.BLUE
			msg_type = "FUNC CALL"
			module = original_func.__module__
			func_name = original_func.__qualname__.split('.')[-1]
			message = f"ARGS[{args}] KWARGS[{kwargs}]"
			self.log_inst.debug( f"!{color}{ module }.py::{  func_name } {color}[{inspect.stack()[1].lineno}] {Fore.GREEN}[{msg_type}]:{Style.RESET_ALL}{message}" )
			
			return original_func(*args, **kwargs)	 
		return wrapper_func

	@classmethod
	def logfunc_cls(cls, logger_attrib_name ):
		call_line_no = inspect.stack()[1].lineno
		def main_decorator(  original_func): 

			def wrapper_func( *args, **kwargs):	 
				log_ref = getattr( args[0],logger_attrib_name, None )
				
				if log_ref: 
					color = Fore.BLUE
					msg_type = "FUNC CALL"
					module = original_func.__module__
					func_name = original_func.__qualname__
					message = f"ARGS[{args}] KWARGS[{kwargs}]"
					# breakpoint()
					log_ref.log_inst.debug( f"?{color}{ module }.py::{  func_name } {color}[{call_line_no}] {Fore.GREEN}[{msg_type}]:{Style.RESET_ALL}{message}" )
				
				return original_func(*args, **kwargs)	 
			return wrapper_func
		return main_decorator
	
	def debug_func(self, module, func_name, message):
		color = Fore.BLUE
		msg_type = "FUNC CALL"
		# breakpoint()
		self.log_inst.debug( f"{color}{ module }::{  func_name } {color}[{inspect.stack()[2].lineno}] {Fore.GREEN}[{msg_type}]:{Style.RESET_ALL}{message}" )

	def debug(self, message): 
		color = Fore.BLUE
		msg_type = "DEBUG"
		# breakpoint()
		self.log_inst.debug( f"{color}{inspect.stack()[1].filename.split('/')[-1]}::{inspect.stack()[1].function}[{inspect.stack()[1].lineno}] [{msg_type}]:{Style.RESET_ALL}{message}" )
	
	def error(self, message): 
		color = Fore.RED
		msg_type = "ERROR"
		self.log_inst.debug( f"{color}{inspect.stack()[1].filename.split('/')[-1]}::{inspect.stack()[1].function}[{inspect.stack()[1].lineno}] [{msg_type}]:{Style.RESET_ALL}{message}" )
	
	def warning(self, message):
		color = Fore.YELLOW
		msg_type = "WARNING"
		self.log_inst.debug( f"{color}{inspect.stack()[1].filename.split('/')[-1]}::{inspect.stack()[1].function}[{inspect.stack()[1].lineno}] [{msg_type}]:{Style.RESET_ALL}{message}" )
	
	def info(self, message): 
		color = Fore.CYAN
		msg_type = "INFO"
		self.log_inst.debug( f"{color}{inspect.stack()[1].filename.split('/')[-1]}::{inspect.stack()[1].function}[{inspect.stack()[1].lineno}] [{msg_type}]:{Style.RESET_ALL}{message}" )
	



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mclogger']

package_data = \
{'': ['*']}

install_requires = \
['colorama>=0.4.4,<0.5.0', 'coloredlogs>=15.0,<16.0', 'tailer>=0.4.1,<0.5.0']

entry_points = \
{'console_scripts': ['mclogger = mclogger.__main__:main']}

setup_kwargs = {
    'name': 'mclogger',
    'version': '1.0.8',
    'description': 'MCLogger that shows log records on screen in color and in a log file',
    'long_description': '# MCLogger: Multi color logger to log to screen and file\n\n\n\n## MCLogger to log to file AND to the console\n\n### What problem does this solve?\nA challenge for web-server applications (e.g. such as Flask) is to decipher what\'s going on from a long logging window.  The standard loggers are all single color console test which you have to trawl through manually one by one.\n\nMCLogger helps to solve this by colorising the debug, info, warning, into different colors so that it is much easier to read.  The logger will output to both on screen and also a file\n\n### How does it do this?\nMCLogger builds on the logging library and adds console color libraries to add colors to debug, info, error, warning entries\n\n* DEBUG - blue\n* WARNING - yellow\n* ERROR - red\n* INFO - cyan\n\n### How to install?\nMCLogger is avaialble through PyPi and you may use pip:\n\n```\n\tpip install mclogger\n```\n\nOr, through git:\n```\n\tgit clone https://github.com/pub12/mclogger.git\n```\n\n\n\n### How to use the logger?\nThe logger is super easy to use.  You need to simply create an instance and add a file/filepath for the logfile\n\n```\nfrom mclogger import MCLogger\n\nfilename = \'log_file.txt\'\nlogger = MultiLogger(filename).getLogger()\n\nlogger.debug("hello world - debug")\nlogger.info("hello world - info")\nlogger.error("hello world - error")\nlogger.warning("hello world - warning")\n```\nOutput will be:\n![Output for mclogger](https://github.com/pub12/mclogger/blob/master/readme/mclogger.png)\n\n### Class Methods overview\n\n- #### getLogger(*filename*):\n\tGenerates the log instance which can be called with .debug(*message*); .info(*message*); .error(*message*); .info(*message*) which will be displayed in color format.\n\n\tFormat of the output message will be: \n\t\t"`<current server timestamp> [<Filename and line number - function name()>] [<DEBUG | INFO | WARN | ERROR >] - < Message >`"\n\n\t- *filename* - filename with optional relative of optional filepath - e.g. `temp/log_file.txt`\n\n- #### read_log_file(*last_n_rows = 20*):\n\tReturn in an array the `last_n_rows` in an array in the order listed within the file.  This will include the color codes for the message\n\n\t- *last_n_rows* - Number of lines to read\n\n- #### read_log_file_as_text(*last_n_rows = 20*):\n\tReturn in an array the `last_n_rows` in an array in the order listed within the file.  This will have color codes removed\n\n\t- *last_n_rows* - Number of lines to read',
    'author': 'Pubs Abayasiri',
    'author_email': 'pubudu.abayasiri@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pubs12/mlogger',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3,<4',
}


setup(**setup_kwargs)
